# -*- coding: utf-8 -*-
import re
from setuptools import setup

REQUIRES = [
    'aiofile',
]

def find_version(fname):
    """Attempts to find the version number in the file names fname.
    Raises RuntimeError if not found.
    """
    version = ""
    with open(fname, "r") as fp:
        reg = re.compile(r'__version__ = [\'"]([^\'"]*)[\'"]')
        for line in fp:
            m = reg.match(line)
            if m:
                version = m.group(1)
                break
    if not version:
        raise RuntimeError("Cannot find version information")
    return version

__version__ = find_version("async_unzip/__init__.py")

setup(
    name="async-unzip",
    version=__version__,
    description="Async unzipping to prevent asyncio timeout errors and decrease the memory usage for bigger zip files",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Dmytro Nikolayev",
    author_email="dmytro@ueni.com",
    url="https://github.com/ueni-ltd/async-unzip",
    packages=["async_unzip"],
    install_requires=REQUIRES,
    license="MIT",
    zip_safe=False,
    keywords="async unzip",
    classifiers=[
        "Intended Audience :: Developers",
        "Environment :: Web Environment",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    requirements = REQUIRES,
    tests_require = ["pytest", "pytest-cov"],
)


from pathlib import PurePath, Path
from zipfile import ZipFile, is_zipfile, BadZipFile
from zlib import decompressobj, MAX_WBITS
from aiofile import async_open

DEFAULT_READ_BUFFER_SIZE = 64 * 1024


async def unzip(zip_file, path=None, files=[], regex_files=None, buffer_size=None, __debug=None):
    read_block = buffer_size if (buffer_size and int(buffer_size)>0) else DEFAULT_READ_BUFFER_SIZE

    if is_zipfile(zip_file):
        files_info = ZipFile(zip_file).infolist()
        if path == None:
            extra_path = ''
        else:
            extra_path = PurePath(path)
        async with async_open(zip_file, 'rb') as src:
            for in_file in files_info:
                file_name = in_file.filename
                unpack_filename_path = Path(str(PurePath(extra_path, file_name)))
                if __debug:
                    print(in_file)
                    print(unpack_filename_path)
                src.seek(in_file.header_offset)
                if __debug:
                    print(f'Done HEADER_OFFSET seek: {in_file.header_offset}')
                await src.read(30)
                if __debug:
                    print(f'Done FILEPATH seek: {30}')
                await src.read(len(in_file.filename))
                if __debug:
                    print(f'Done FILENAME seek: {len(in_file.filename)}')
                if len(in_file.extra) > 0:
                    t = await src.read(len(in_file.extra))
                    if __debug:
                        print(f'Done EXTRA seek: {len(in_file.extra)} {t}')
                if len(in_file.comment) > 0:
                    await src.read(len(in_file.comment))
                    if __debug:
                        print(f'Done COMMENT seek: {len(in_file.comment)}')

                if in_file.is_dir():
                    unpack_filename_path.mkdir(parents=True, exist_ok=True)
                    continue
                else:
                    unpack_filename_path.parent.mkdir(parents=True, exist_ok=True)


                async with async_open(str(unpack_filename_path), 'wb+') as out:
                    i = in_file.compress_size
                    decomp = decompressobj(-MAX_WBITS)

                    buf = await src.read(read_block)
                    if __debug:
                        print(f'Length: {len(buf)}')
                    while buf:
                        result = decomp.decompress(buf)
                        await out.write(result)
                        curr_read_block = read_block if i > read_block else i
                        buf = await src.read(curr_read_block)
                        i -= curr_read_block
                        if __debug:
                            print(f'Length: {len(buf)}')

                    result = decomp.flush()
                    if __debug:
                        print(f'Flush Length: {len(buf)}')
                    await out.write(result)
    else:
        raise BadZipFile
